#Monitored restart function
#mjolinor 05/16/2010

function monitored-restart {
param ($server = $(throw "A server name is required."))

$events = @()
$events += "Started restart attempt on $server at $((get-date).tolongtimestring())`n" 
Write-Host  "Started restart attempt on $server at $((get-date).tolongtimestring())`n" 


$pingfail_timeout = 180
$pingresume_timeout = 180
$servicecheck_delay = 180

function Test-Port{
    Param([string]$srv,$port=135,$timeout=1000,[switch]$verbose)
    $ErrorActionPreference = "SilentlyContinue"
    $tcpclient = new-Object system.Net.Sockets.TcpClient
    $iar = $tcpclient.BeginConnect($srv,$port,$null,$null)
    $wait = $iar.AsyncWaitHandle.WaitOne($timeout,$false)
    if(!$wait)
    {
        $tcpclient.Close()
        if($verbose){Write-Host "Connection Timeout"}
        Return $false
    }
    else
    {
        $error.Clear()
        $tcpclient.EndConnect($iar) | out-Null
        if($error[0]){if($verbose){write-host $error[0]};$failed = $true}
        $tcpclient.Close()
    }   
    if($failed){return $false}else{return $true}
}

try {Restart-Computer  $server -force -ErrorAction Stop}
catch {
	$events += "Restart command was not accepted by $server`n"
	write-host "Restart command was not accepted by $server`n" 
	$restart_failed = $true
	}

if (-not ($restart_failed)){
$events += "Restart request was accepted by the server at $((get-date).tolongtimestring())`n" 
write-host "Restart request was accepted by the server at $((get-date).tolongtimestring())`n" 
Write-Host "Waiting on RPC ping fail...`n"

$timer = [System.Diagnostics.Stopwatch]::StartNew()

$timer.reset()
$timer.start()

while ($timer.elapsed.totalseconds -lt $pingfail_timeout){
if(-not (test-port ($server))){
	$events += "RPC ping fail detected at $((get-date).tolongtimestring())`n" 
	write-host "RPC ping fail detected at $((get-date).tolongtimestring())`n"
	$RPC_pingfail = $true
	break
	}
}

Write-Host "Waiting on ICMP ping fail...`n"
while ($timer.elapsed.totalseconds -lt $pingfail_timeout){
if (-not (test-connection $server -Quiet)) {
	$events += "ICMP ping fail detected at $((get-date).tolongtimestring())`n" 
	write-host "ICMP ping fail detected at $((get-date).tolongtimestring())`n"
	$pingfail = $true
	break
	}
}

if (-not($pingfail)){
	$events += "ICMP ping fail not detected in $pingfail_timeout seconds.  Server appears to be hung.`n"
	write-host "ICMP ping fail not detected in $pingfail_timeout seconds.  Server appears to be hung.`n" 
	exit
	}

Write-Host "Waiting on ping resume..."`n
$timer.reset()
$timer.start()
While ($timer.elapsed.totalseconds -lt $pingresume_timeout){
if (test-connection $server -Quiet){
	$events +=  "ICMP ping resumed at $((get-date).tolongtimestring())`n" 
	write-host  "ICMP ping resumed at $((get-date).tolongtimestring())`n"
	$ping_resume = $true
	break
	}
}

while ($timer.elapsed.totalseconds -lt $pingresume_timeout){
if(test-port -server $server){
	$events += "RPC ping resumed at $((get-date).tolongtimestring())`n" 
	write-host "RPC ping resumed at $((get-date).tolongtimestring())`n"
	$RPC_pingresume = $true
	break
	}
}

if (-not($ping_resume)){
	$events += "ICMP ping resume not detected in $pingresume_timeout seconds.  Server appears to be hung.`n"
	write-host "ICMP ping resume not detected in $pingresume_timeout seconds.  Server appears to be hung.`n"
		}

	if ($ping_resume -and -not $rpc_pingresume){
	$events += "RPC ping resume not detected in $pingresume_timeout seconds.  Server appears to be hung.`n"
	write-host "RPC ping resume not detected in $pingresume_timeout seconds.  Server appears to be hung.`n"
	exit
	}
	
$events +=  "Waiting $servicecheck_delay seconds for service startup...`n"
write-host  "Waiting $servicecheck_delay seconds for service startup...`n"

sleep -Seconds $servicecheck_delay

$events += gwmi win32_service -ComputerName $server |? {$_.startmode -eq "auto"} | ft state,name,status -auto | out-string
}

$events

}

monitored-restart ($args[0])